@@ -2,7 +2,7 @@
 Asunder
 
 Copyright(C) 2005 Eric Lathrop <eric@ericlathrop.com>
-Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/misc/contactandrew.php>
+Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/contact.php>
 
 Any code in this file may be redistributed or modified under the terms of
 the GNU General Public Licence as published by the Free Software 
@@ -57,13 +57,6 @@ format_wavpack_bitrate                 (GtkScale *scale,
     return g_strdup_printf (_("%dKbps"), int_to_wavpack_bitrate((int)arg1));
 }
 
-gboolean
-idle(gpointer data)
-{
-    refresh(global_prefs->cdrom, 0);
-    return (data != NULL);
-}
-
 void
 on_about_clicked                       (GtkToolButton   *toolbutton,
                                         gpointer         user_data)
@@ -313,7 +306,12 @@ void
 on_pick_disc_changed                   (GtkComboBox     *combobox,
                                         gpointer         user_data)
 {
-    cddb_disc_t * disc = g_list_nth_data(gbl_disc_matches, gtk_combo_box_get_active(combobox));
+    gint selected = gtk_combo_box_get_active(combobox);
+    if (gbl_disc_matches == NULL ||
+        selected == -1 ||
+        (guint)selected >= g_list_length(gbl_disc_matches))
+        return;
+    cddb_disc_t * disc = g_list_nth_data(gbl_disc_matches, selected);
     update_tracklist(disc);
 }
 
@@ -392,9 +390,7 @@ on_lookup_clicked                     (GtkToolButton   *toolbutton,
        if (!thread_helpers_in_main_thread ()) gdk_threads_leave ();
     }
     */
-    gdk_threads_leave();
-    refresh(global_prefs->cdrom, 1);
-    gdk_threads_enter();
+    refresh();
 }
 
 void
@@ -721,6 +717,27 @@ on_tracklist_mouse_click               (GtkWidget* treeView,
     return FALSE;
 }
 
+void
+on_rip_header_click                 (GtkTreeViewColumn *treeviewcolumn,
+                                     gpointer           user_data)
+{
+    // Select/Deselect all tracks.
+    // Get first checkbox and set all to opposite value.
+    GtkTreeIter iter;
+    int riptrack;
+    GtkTreeModel * model = gtk_tree_view_get_model(
+        GTK_TREE_VIEW(lookup_widget(win_main, "tracklist")));
+
+    if (!gtk_tree_model_get_iter_first(model, &iter))
+        return;
+    gtk_tree_model_get(model, &iter, COL_RIPTRACK, &riptrack, -1);
+
+    if (riptrack)
+        gtk_tree_model_foreach(model, for_each_row_deselect, NULL);
+    else
+        gtk_tree_model_foreach(model, for_each_row_select, NULL);
+}
+
 void
 on_window_close                        (GtkWidget       *widget,
                                         GdkEventFocus   *event,
