@@ -18,10 +18,6 @@ format_wavpack_bitrate                 (GtkScale *scale,
                                         gdouble   arg1,
                                         gpointer  user_data);
 
-gboolean
-idle(gpointer data);
-
-
 void
 on_about_clicked                       (GtkToolButton   *toolbutton,
                                         gpointer         user_data);
@@ -201,6 +197,10 @@ on_tracklist_mouse_click               (GtkWidget* treeView,
                                         GdkEventButton* event, 
                                         gpointer user_data);
 
+void
+on_rip_header_click                 (GtkTreeViewColumn *treeviewcolumn,
+                                     gpointer           user_data);
+
 void
 on_window_close	                       (GtkWidget       *widget,
                                         GdkEventFocus   *event,
