@@ -2,7 +2,7 @@
 Asunder
 
 Copyright(C) 2005 Eric Lathrop <eric@ericlathrop.com>
-Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/misc/contactandrew.php>
+Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/contact.php>
 
 Any code in this file may be redistributed or modified under the terms of
 the GNU General Public Licence as published by the Free Software 
@@ -1438,14 +1438,14 @@ void enable_musepack_widgets(void)
 }
 
 const char* 
-GBLprogramName = "Asunder 2.8.1";
+GBLprogramName = "Asunder 2.9.1";
 
 static const char* 
 GBLauthors[2] = {
 "Many thanks to all the following people:\n"
 "\n"
 "Andrew Smith\n"
-"http://littlesvr.ca/misc/contactandrew.php\n"
+"http://littlesvr.ca/contact.php\n"
 "Summer 2005 - 2009\n"
 "- maintainer\n"
 "\n"
@@ -1454,6 +1454,25 @@ GBLauthors[2] = {
 "- original author\n"
 "- 'eject when finished' feature\n"
 "\n"
+"Gregory Margo\n"
+"- Shorten requested filenames that are longer than the filesystem can handle\n"
+"- Work around cdparanoia's filename length limits\n"
+"- Fixed bug where multiple CDDB entries would only show once\n"
+"- Improved the refresh/lookup system to work much better with more complicated CDs\n"
+"- Fixed FLAC progress scanner. Both to make the percentage more reliable and to prevent a possible crash\n"
+"- Added feature to select/deselect all tracks by clicking on rip checkbox header\n"
+"- Fixed autocomplete file loading\n"
+"- Fixed several potential buffer overflow bugs\n"
+"- Fixed some minor memory leaks\n"
+"- Fixed race condition during abort\n"
+"- Improved completion percentage feedback\n"
+"- Added feature to make the artist and title columns resizable\n"
+"(A few patches were mistakenly attributed to \"Gregory Montego\" and \"Gergory Margo\")\n"
+"\n"
+"Andreas Ronnquist\n"
+"- Moved the config file to ~/.config/asunder without breaking old configs\n"
+"- Moved the autocomplete files to ~/.cache/asunder without breaking old caches\n"
+"\n"
 "Fraser Tweedale\n"
 "- FreeBSD port\n"
 "\n"
@@ -1500,6 +1519,15 @@ GBLauthors[2] = {
 "Nicolas Leveille\n"
 "- Fix for: don't reset the track info edited by the user by doing an unnecessary CDDB lookup.\n"
 "\n"
+"Pierre Lestringant (whz)\n"
+"- Fixed musepack and aac playlist generation\n"
+"\n"
+"Matthew (mw)\n"
+"- Add the ALBUMARTIST tag to flac files when \"Single artist\" is not checked.\n"
+"\n"
+"Antony Gelberg\n"
+"- Fixed when the CD is ejected to avoid problems before a full rip/encode cycle is completed.\n"
+"\n"
 "Packages:\n"
 "\n"
 "Toni Graffy\n"
@@ -1777,7 +1805,7 @@ N_("An application to save tracks from an Audio CD \n"
 static const char* 
 GBLcopyright = 
 "Copyright 2005 Eric Lathrop\n"
-"Copyright 2007 - 2012 Andrew Smith";
+"Copyright 2007 - 2017 Andrew Smith";
 
 static const char* 
 GBLwebsite = "http://littlesvr.ca/asunder/";
@@ -1805,19 +1833,21 @@ show_aboutbox (void)
 void show_completed_dialog(int numOk, int numFailed)
 {
     GtkWidget* dialog;
-    
-    dialog = gtk_message_dialog_new(GTK_WINDOW(win_main),
-                                    GTK_DIALOG_DESTROY_WITH_PARENT,
-                                    GTK_MESSAGE_INFO,
-                                    GTK_BUTTONS_CLOSE,
-                                    ngettext("%d file created successfully", "%d files created successfully", numOk),
-                                    numOk);
-    
-    if (numFailed > 0)
+
+    if (numFailed == 0)
     {
         dialog = gtk_message_dialog_new(GTK_WINDOW(win_main),
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
-                                        GTK_MESSAGE_WARNING,
+                                        GTK_MESSAGE_INFO,
+                                        GTK_BUTTONS_CLOSE,
+                                        ngettext("%d file created successfully", "%d files created successfully", numOk),
+                                        numOk);
+    }
+    else
+    {
+        dialog = gtk_message_dialog_new(GTK_WINDOW(win_main),
+                                        GTK_DIALOG_DESTROY_WITH_PARENT,
+                                        GTK_MESSAGE_ERROR,
                                         GTK_BUTTONS_CLOSE,
                                         ngettext("There was an error creating %d file", "There was an error creating %d files", numFailed),
                                         numFailed);
