@@ -2,7 +2,7 @@
 Asunder
 
 Copyright(C) 2005 Eric Lathrop <eric@ericlathrop.com>
-Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/misc/contactandrew.php>
+Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/contact.php>
 
 Any code in this file may be redistributed or modified under the terms of
 the GNU General Public Licence as published by the Free Software 
@@ -61,6 +61,12 @@ GtkWidget * album_year;
 
 GtkWidget * tracklist;
 GtkWidget * pick_disc;
+GtkWidget * statusLbl;
+
+static GMutex refresh_mutex;
+static GCond refresh_cond;
+static int refresh_forced = 0;      /* modify only while protected by mutex */
+static gboolean start_refresh_thread(gpointer data);
 
 extern const char* GBLprogramName;
 
@@ -119,6 +125,7 @@ int main(int argc, char *argv[])
     album_year = lookup_widget(win_main, "album_year");
     tracklist = lookup_widget(win_main, "tracklist");
     pick_disc = lookup_widget(win_main, "pick_disc");
+    statusLbl = lookup_widget(win_main, "statusLbl");
     
     // set up all the columns for the track listing widget
     renderer = gtk_cell_renderer_toggle_new();
@@ -126,6 +133,9 @@ int main(int argc, char *argv[])
     g_signal_connect(renderer, "toggled", (GCallback) on_rip_toggled, NULL);
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Rip"), renderer, "active", COL_RIPTRACK, NULL);
+    GtkTreeViewColumn * col = gtk_tree_view_get_column(GTK_TREE_VIEW(tracklist), COL_RIPTRACK);
+    gtk_tree_view_column_set_clickable(col, TRUE);
+    g_signal_connect((gpointer)col, "clicked", G_CALLBACK(on_rip_header_click), NULL);
 
     renderer = gtk_cell_renderer_text_new();
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
@@ -136,12 +146,16 @@ int main(int argc, char *argv[])
     g_signal_connect(renderer, "edited", (GCallback) on_artist_edited, NULL);
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Artist"), renderer, "text", COL_TRACKARTIST, NULL);
+    col = gtk_tree_view_get_column(GTK_TREE_VIEW(tracklist), COL_TRACKARTIST);
+    gtk_tree_view_column_set_resizable(col, TRUE);
 
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "editable", TRUE, NULL);
     g_signal_connect(renderer, "edited", (GCallback) on_title_edited, NULL);
     gtk_tree_view_insert_column_with_attributes(GTK_TREE_VIEW(tracklist), -1, 
                     _("Title"), renderer, "text", COL_TRACKTITLE, NULL);
+    col = gtk_tree_view_get_column(GTK_TREE_VIEW(tracklist), COL_TRACKTITLE);
+    gtk_tree_view_column_set_resizable(col, TRUE);
 
     renderer = gtk_cell_renderer_text_new();
     g_object_set(renderer, "editable", TRUE, NULL);
@@ -185,10 +199,8 @@ int main(int argc, char *argv[])
     
     gtk_widget_show(win_main);
     
-    // set up recurring timeout to automatically re-scan the cdrom once every second
-    g_timeout_add(500, idle, (void *)1);
-    // add an idle event to scan the cdrom drive ASAP
-    g_idle_add(idle, NULL);
+    // Start the refresh thread after gtk startup.
+    g_idle_add(start_refresh_thread, NULL);
 
     gdk_threads_enter();
     gtk_main();
@@ -204,40 +216,56 @@ bool check_disc(char * cdrom)
     int fd;
     bool ret = false;
     int status;
+    char msgStr[1024];
+    gint64 diff_time, open_diff_time;
+    gint64 start_time = g_get_monotonic_time();
+
+    static bool nowBusy = true;  /* more feedback if check_disc is taking a long time */
+    bool newBusy = false;
+#define BUSY_THRESHOLD  (2*1000*1000)
     
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__) || defined(__NetBSD__)
     struct ioc_read_subchannel cdsc;
     struct cd_sub_channel_info data;
 #endif
     
+    if (nowBusy)
+    {
+        // The big problem is if the disc has been changed.
+        // The open could take a long time without a status message.
+        gdk_threads_enter();
+        gtk_label_set_markup(GTK_LABEL(statusLbl), _("<b>Checking disc...</b>"));
+        gdk_threads_leave();
+    }
+    
     // open the device
     fd = open(cdrom, O_RDONLY | O_NONBLOCK);
-    if (fd < 0)
+    open_diff_time = g_get_monotonic_time() - start_time;
+    diff_time = open_diff_time;
+    if (fd < 0 || working || refresh_forced)
     {
-        fprintf(stderr, "Error: Couldn't open %s\n", cdrom);
+        if (fd < 0)
+            fprintf(stderr, "Error: Couldn't open %s\n", cdrom);
+        if (nowBusy)
+        {
+            gdk_threads_enter();
+            gtk_label_set_text(GTK_LABEL(statusLbl), "");
+            gdk_threads_leave();
+        }
+        nowBusy = diff_time > BUSY_THRESHOLD;
         return false;
     }
     
-    /* this was the original (Eric's 0.1 and post 0.0.1) checking code,
-    * but it never worked properly for me. Replaced 21 aug 2007. */
-    //~ static bool newdisc = true;
-    //~ // read the drive status info
-    //~ if (ioctl(fd, CDROM_DRIVE_STATUS, CDSL_CURRENT) == CDS_DISC_OK)
-    //~ {
-        //~ if (newdisc)
-        //~ {
-            //~ newdisc = false;
-            
-            //~ status = ioctl(fd, CDROM_DISC_STATUS, CDSL_CURRENT);
-            //~ if ((status == CDS_AUDIO) || (status == CDS_MIXED))
-            //~ {
-                //~ ret = true;
-            //~ }printf("status %d vs %d\n", status, CDS_NO_INFO);
-        //~ }
-    //~ } else {
-        //~ newdisc = true;
-        //~ clear_widgets();
-    //~ }
+    // Debug print add time last taken by the open() call.
+    //snprintf(msgStr, 1024, "%s [open %.1lf sec]",
+        //_("<b>Checking disc...</b>"), open_diff_time / 1E6);
+    snprintf(msgStr, 1024, "%s", _("<b>Checking disc...</b>"));
+    if (nowBusy || diff_time > BUSY_THRESHOLD)
+    {
+        gdk_threads_enter();
+        gtk_label_set_markup(GTK_LABEL(statusLbl), msgStr);
+        gdk_threads_leave();
+    }
     
     static bool alreadyKnowGood = false; /* check when program just started */
     static bool alreadyCleared = true; /* no need to clear when program just started */
@@ -260,6 +288,7 @@ bool check_disc(char * cdrom)
             ret = true;
             alreadyKnowGood = true; /* don't return true again for this disc */
             alreadyCleared = false; /* clear when disc is removed */
+            newBusy = true;
         }
     }
     else
@@ -268,11 +297,40 @@ bool check_disc(char * cdrom)
         if (!alreadyCleared)
         {
             alreadyCleared = true;
+            gdk_threads_enter();
             clear_widgets();
+            gdk_threads_leave();
+            newBusy = true;
         }
     }
 
     close(fd);
+    
+    diff_time = g_get_monotonic_time() - start_time;
+    if (nowBusy || diff_time > BUSY_THRESHOLD)
+    {
+        // Debug print last time taken to do the ioctl()
+        //int len = strlen(msgStr);
+        //snprintf(&msgStr[len], 1024-len, " [status %.1lf sec]", (diff_time - open_diff_time) / 1E6);
+        //gdk_threads_enter();
+        //gtk_label_set_markup(GTK_LABEL(statusLbl), msgStr);
+        //gdk_threads_leave();
+    }
+    else
+    {
+        gdk_threads_enter();
+        gtk_label_set_text(GTK_LABEL(statusLbl), "");
+        gdk_threads_leave();
+    }
+    nowBusy = newBusy || diff_time > BUSY_THRESHOLD;
+    
+    if (working || refresh_forced)
+    {
+        gdk_threads_enter();
+        gtk_label_set_text(GTK_LABEL(statusLbl), "");
+        gdk_threads_leave();
+    }
+    
     return ret;
 }
 
@@ -392,100 +450,103 @@ void eject_disc(char * cdrom)
     }
 }
 
-static GThread * gbl_cddb_query_thread;
-static int gbl_cddb_query_thread_running;
-static cddb_conn_t * gbl_cddb_query_thread_conn;
-static cddb_disc_t * gbl_cddb_query_thread_disc;
-static int gbl_cddb_query_thread_num_matches;
-static GList * gbl_matches = NULL;
-
-gpointer cddb_query_thread_run(gpointer data)
+GList * cddb_query_run(cddb_conn_t * conn, cddb_disc_t * original_disc)
 {
     char logStr[1024];
     int i;
+    GList * matches = NULL;
+    // Use clone of original disc since cddb_query_next() will overwrite it.
+    cddb_disc_t * disc = cddb_disc_clone(original_disc);
     
-    gbl_cddb_query_thread_num_matches = cddb_query(gbl_cddb_query_thread_conn, gbl_cddb_query_thread_disc);
-    if(gbl_cddb_query_thread_num_matches == -1)
-        gbl_cddb_query_thread_num_matches = 0;
+    int num_matches = cddb_query(conn, disc);
+    if (num_matches == -1)
+    {
+        cddb_error_print(cddb_errno(conn));
+        num_matches = 0;
+    }
     
-    snprintf(logStr, 1024, "Found %d CDDB matches", gbl_cddb_query_thread_num_matches);
+    snprintf(logStr, 1024, "Found %d CDDB matches", num_matches);
     debugLog(logStr);
     
-    gbl_matches = NULL;
-    
     // make a list of all the matches
-    for (i = 0; i < gbl_cddb_query_thread_num_matches; i++)
+    for (i = 0; i < num_matches; i++)
     {
-        cddb_disc_t * possible_match = cddb_disc_clone(gbl_cddb_query_thread_disc);
-        if (cddb_read(gbl_cddb_query_thread_conn, possible_match) == 1)
+        cddb_disc_t * possible_match = cddb_disc_clone(disc);
+        if (cddb_read(conn, possible_match))
         {
-            gbl_matches = g_list_append(gbl_matches, possible_match);
+            snprintf(logStr, 1024, "Match %d: '%08x' '%s' '%s' '%s'\n",
+                i + 1,
+                cddb_disc_get_discid(possible_match),
+                cddb_disc_get_artist(possible_match),
+                cddb_disc_get_title(possible_match),
+                cddb_disc_get_genre(possible_match));
+            debugLog(logStr);
+
+            matches = g_list_append(matches, possible_match);
             
             // move to next match
-            if (i < gbl_cddb_query_thread_num_matches - 1)
+            if (i < num_matches - 1)
             {
-                if (!cddb_query_next(gbl_cddb_query_thread_conn, gbl_cddb_query_thread_disc))
+                if (!cddb_query_next(conn, disc))
                     fatalError("Query index out of bounds.");
             }
         }
         else
-            printf("Failed to cddb_read()\n");
+        {
+            fprintf(stderr, "Failed to cddb_read()\n");
+            cddb_error_print(cddb_errno(conn));
+            cddb_disc_destroy(possible_match);
+        }
     }
     
-    g_atomic_int_set(&gbl_cddb_query_thread_running, 0);
-    
-    return NULL;
+    cddb_disc_destroy(disc);
+    return matches;
 }
 
 GList * lookup_disc(cddb_disc_t * disc)
 {
     // set up the connection to the cddb server
-    gbl_cddb_query_thread_conn = cddb_new();
-    if (gbl_cddb_query_thread_conn == NULL)
+    cddb_conn_t * conn = cddb_new();
+    if (conn == NULL)
         fatalError("cddb_new() failed. Out of memory?");
     
-    cddb_set_server_name(gbl_cddb_query_thread_conn, global_prefs->cddb_server_name);
-    cddb_set_server_port(gbl_cddb_query_thread_conn, global_prefs->cddb_port_number);
+    cddb_set_server_name(conn, global_prefs->cddb_server_name);
+    cddb_set_server_port(conn, global_prefs->cddb_port_number);
     
     if (global_prefs->use_proxy)
     {
-        cddb_set_http_proxy_server_name(gbl_cddb_query_thread_conn, global_prefs->server_name);
-        cddb_set_http_proxy_server_port(gbl_cddb_query_thread_conn, global_prefs->port_number);
-        cddb_http_proxy_enable(gbl_cddb_query_thread_conn);
+        cddb_set_http_proxy_server_name(conn, global_prefs->server_name);
+        cddb_set_http_proxy_server_port(conn, global_prefs->port_number);
+        cddb_http_proxy_enable(conn);
     }
     
     // force HTTP when port 80 (for MusicBrainz). This code by Tim.
     if (global_prefs->cddb_port_number == 80)
-        cddb_http_enable(gbl_cddb_query_thread_conn);
-    
-    // query cddb to find similar discs
-    g_atomic_int_set(&gbl_cddb_query_thread_running, 1);
-    gbl_cddb_query_thread_disc = disc;
-    gbl_cddb_query_thread = g_thread_create(cddb_query_thread_run, NULL, TRUE, NULL);
+        cddb_http_enable(conn);
+
+    // Disable caching of CDDB entries since libcddb bug fouls up multiple matches.
+    // https://sourceforge.net/p/libcddb/bugs/9/
+    cddb_cache_disable(conn);
     
     // show cddb update window
     gdk_threads_enter();
         disable_all_main_widgets();
         
-        GtkWidget* statusLbl = lookup_widget(win_main, "statusLbl");
-        gtk_label_set_text(GTK_LABEL(statusLbl), _("<b>Getting disc info from the internet...</b>"));
-        gtk_label_set_use_markup(GTK_LABEL(statusLbl), TRUE);
-        
-        while(g_atomic_int_get(&gbl_cddb_query_thread_running) != 0)
-        {
-            while (gtk_events_pending())
-                gtk_main_iteration();
-            usleep(100000);
-        }
-        
+        gtk_label_set_markup(GTK_LABEL(statusLbl), _("<b>Getting disc info from the internet...</b>"));
+    gdk_threads_leave();
+
+    // query cddb to find similar discs
+    GList * matches = cddb_query_run(conn, disc);
+
+    gdk_threads_enter();
         gtk_label_set_text(GTK_LABEL(statusLbl), "");
         
         enable_all_main_widgets();
     gdk_threads_leave();
     
-    cddb_destroy(gbl_cddb_query_thread_conn);
+    cddb_destroy(conn);
     
-    return gbl_matches;
+    return matches;
 }
 
 // reads the TOC of a cdrom into a CDDB struct
@@ -525,6 +586,15 @@ cddb_disc_t * read_disc(char * cdrom)
         fprintf(stderr, "Error: Couldn't open %s\n", cdrom);
         return NULL;
     }
+
+    // Add indicator that read_disc() is occuring since it might take a long time.
+    gdk_threads_enter();
+    // Should we disable all widgets here, like lookup_disc() does?
+    // Might it create a flicker?
+    // Seems hardly noticable for "normal" discs.
+    disable_all_main_widgets();
+    gtk_label_set_markup(GTK_LABEL(statusLbl), _("<b>Reading disc...</b>"));
+    gdk_threads_leave();
     
 #if defined(__FreeBSD__) || defined(__FreeBSD_kernel__)
     // read disc status info
@@ -570,6 +640,8 @@ cddb_disc_t * read_disc(char * cdrom)
                     cddb_track_set_artist(track, "Unknown Artist");
                     cddb_disc_add_track(disc, track);
                 }
+                if (working || refresh_forced)
+                    break;
             }
             te.track = 0xAA;
             if (ioctl(fd, CDIOREADTOCENTRY, &te) == 0)
@@ -624,6 +696,8 @@ cddb_disc_t * read_disc(char * cdrom)
                     cddb_track_set_artist(track, "Unknown Artist");
                     cddb_disc_add_track(disc, track);
                 }
+                if (working || refresh_forced)
+                    break;
             }
             te.starting_track = 0xAA;
             if (ioctl(fd, CDIOREADTOCENTRIES, &te) == 0)
@@ -671,6 +745,8 @@ cddb_disc_t * read_disc(char * cdrom)
                     cddb_track_set_artist(track, "Unknown Artist");
                     cddb_disc_add_track(disc, track);
                 }
+                if (working || refresh_forced)
+                    break;
             }
             
             te.cdte_track = CDROM_LEADOUT;
@@ -686,7 +762,22 @@ cddb_disc_t * read_disc(char * cdrom)
     /* These two lines from Nicolas Lveill
     * "let us have a discid for each read disc" */
     if (disc)
+    {
         cddb_disc_calc_discid(disc);
+        snprintf(logStr, 1024, "read_disc: discid=%08x\n", cddb_disc_get_discid(disc));
+        debugLog(logStr);
+    }
+
+    gdk_threads_enter();
+    gtk_label_set_text(GTK_LABEL(statusLbl), "");
+    enable_all_main_widgets();
+    gdk_threads_leave();
+
+    if (working || refresh_forced)
+    {
+        cddb_disc_destroy(disc);
+        disc = NULL;
+    }
 
     return disc;
 }
@@ -705,7 +796,8 @@ void update_tracklist(cddb_disc_t * disc)
     
     gbl_current_discid = cddb_disc_get_discid(disc);
 
-    sprintf(logStr, "update_tracklist() disk '%s' '%s' '%s'\n", disc_artist, disc_title, disc_genre);
+    snprintf(logStr, 1024, "update_tracklist() disk '%08x' '%s' '%s' '%s'\n",
+        gbl_current_discid, disc_artist, disc_title, disc_genre);
     debugLog(logStr);
     if (disc_artist != NULL)
     {
@@ -756,10 +848,23 @@ void update_tracklist(cddb_disc_t * disc)
 }
 
 
-void refresh(char * cdrom, int force)
+void refresh(void)
+{
+    // This is to deal with the fact that the signal below will take up to
+    // a second to get processed in refresh_thread() which will call check_disk()
+    // which will set the label to the same thing at the start.
+    gtk_label_set_markup(GTK_LABEL(statusLbl), _("<b>Checking disc...</b>"));
+    
+    /* Wake up the refresh thread. */
+    g_mutex_lock(&refresh_mutex);
+    refresh_forced = 1;
+    g_cond_signal(&refresh_cond);
+    g_mutex_unlock(&refresh_mutex);
+}
+
+void refresh_thread_body(char * cdrom, int force)
 {
     cddb_disc_t * disc;
-    //GList * curr;
     
     if(working)
     /* don't do nothing */
@@ -767,6 +872,9 @@ void refresh(char * cdrom, int force)
     
     if (check_disc(cdrom) || force)
     {
+        if (working || refresh_forced)
+            return;
+
         disc = read_disc(cdrom);
         if (disc == NULL)
             return;
@@ -775,39 +883,47 @@ void refresh(char * cdrom, int force)
         {
             /* only trash the user's inputs when the disc is new */
             
+            gdk_threads_enter();
             gtk_widget_set_sensitive(lookup_widget(win_main, "rip_button"), TRUE);
         
             // show the temporary info
             gtk_entry_set_text(GTK_ENTRY(album_artist), "Unknown Artist");
             gtk_entry_set_text(GTK_ENTRY(album_title), "Unknown Album");
             update_tracklist(disc);
+            gdk_threads_leave();
         }
         
-        // clear out the previous list of matches
-        /* this causes a segfault in the following scenario:
-        - disable auto cddb lookup
-        - insert cd that has a record in cddb
-        - click 'cddb lookup'
-        - wait for lookup to finish successfully and eject the disk
-        - reinsert the disk
-        - click 'cddb lookup'
-        - it crashes in the following loop (legrand-sw 14 sep 2008)
-        for (curr = g_list_first(gbl_disc_matches); curr != NULL; curr = g_list_next(curr))
+        if ((!global_prefs->do_cddb_updates && !force) || working || refresh_forced)
         {
-            cddb_disc_destroy((cddb_disc_t *)curr->data);
+            cddb_disc_destroy(disc);
+            return;
         }
-        g_list_free(gbl_disc_matches);
-        */
         
-        if (!global_prefs->do_cddb_updates && !force)
-            return;
-        
-        gbl_disc_matches = lookup_disc(disc);
+        GList * disc_matches = lookup_disc(disc);
         cddb_disc_destroy(disc);
-        
-        if (gbl_disc_matches == NULL)
+
+        if (working || refresh_forced)
             return;
         
+        // Only access/modify gbl_disc_matches inside gdk_thread lock, or else
+        // another mutex is needed.
+        gdk_threads_enter();
+
+        // clear out the previous list of matches
+        if (gbl_disc_matches != NULL)
+        {
+            for (GList * curr = g_list_first(gbl_disc_matches); curr != NULL; curr = g_list_next(curr))
+                cddb_disc_destroy((cddb_disc_t *)curr->data);
+            g_list_free(gbl_disc_matches);
+        }
+        gbl_disc_matches = disc_matches;
+
+        if (gbl_disc_matches == NULL)
+        {
+            gdk_threads_leave();
+            return;
+        }
+        
         if (g_list_length(gbl_disc_matches) > 1)
         {
             // fill in and show the album drop-down box
@@ -826,13 +942,49 @@ void refresh(char * cdrom, int force)
                     -1);
             }
             gtk_combo_box_set_model(GTK_COMBO_BOX(pick_disc), GTK_TREE_MODEL(store));
-            gtk_combo_box_set_active(GTK_COMBO_BOX(pick_disc), 1);
+            g_object_unref(store);
             gtk_combo_box_set_active(GTK_COMBO_BOX(pick_disc), 0);
             
             gtk_widget_show(lookup_widget(win_main, "disc"));
             gtk_widget_show(lookup_widget(win_main, "pick_disc"));
         }
-        
-        update_tracklist((cddb_disc_t *)g_list_nth_data(gbl_disc_matches, 0));
+        else
+            update_tracklist((cddb_disc_t *)g_list_nth_data(gbl_disc_matches, 0));
+        gdk_threads_leave();
     }
 }
+
+static gpointer refresh_thread(gpointer data)
+{
+    gint64 end_time;
+    int force = 0;
+
+    while (1)
+    {
+        refresh_thread_body(global_prefs->cdrom, force);
+
+        /* Wait X ms before next iteration, or until awakened. */
+        g_mutex_lock(&refresh_mutex);
+
+        // 500ms is too flickery for my taste, especially if status messages are up
+        //end_time = g_get_monotonic_time() + 500 * G_TIME_SPAN_MILLISECOND;
+        end_time = g_get_monotonic_time() + 1000 * G_TIME_SPAN_MILLISECOND;
+
+        while (!refresh_forced)
+            if (!g_cond_wait_until(&refresh_cond, &refresh_mutex, end_time))
+                break; /* timeout has passed */
+
+        force = refresh_forced;
+        refresh_forced = 0;
+
+        g_mutex_unlock(&refresh_mutex);
+    }
+
+    return(NULL);
+}
+
+static gboolean start_refresh_thread(gpointer data)
+{
+    g_thread_create(refresh_thread, NULL, FALSE, NULL);
+    return(FALSE);
+}
