@@ -2,7 +2,7 @@
 Asunder
 
 Copyright(C) 2005 Eric Lathrop <eric@ericlathrop.com>
-Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/misc/contactandrew.php>
+Copyright(C) 2007 Andrew Smith <http://littlesvr.ca/contact.php>
 
 Any code in this file may be redistributed or modified under the terms of
 the GNU General Public Licence as published by the Free Software 
@@ -216,11 +216,12 @@ void sigchld(int signum)
 // args - a valid array for execvp()
 // toread - the file descriptor to pipe back to the parent
 // p - a place to write the PID of the exec'ed process
+// dir - directory to run program in
 // 
 // returns - a file descriptor that reads whatever the program outputs on "toread"
-int exec_with_output(const char * args[], int toread, pid_t * p)
+int exec_with_output(const char * args[], int toread, pid_t * p, const char * dir)
 {
-    char logStr[1024];
+    char logStr[2048];
     int pipefd[2];
     
     blockSigChld();
@@ -255,6 +256,14 @@ int exec_with_output(const char * args[], int toread, pid_t * p)
         dup2(pipefd[1], toread);
         close(pipefd[1]);
         
+        // Change directory if required
+        if (dir && (chdir(dir) < 0))
+        {
+            snprintf(logStr, 2048, "Failed to change directory for %s, errno=%d", args[0], errno);
+            debugLog(logStr);
+            exit(0);
+        }
+        
         // call execvp
         execvp(args[0], (char **)args);
         
@@ -263,10 +272,10 @@ int exec_with_output(const char * args[], int toread, pid_t * p)
     }
     
     int count;
-    snprintf(logStr, 1024, "%d started: %s ", *p, args[0]);
+    snprintf(logStr, 2048, "%d started: %s ", *p, args[0]);
     for (count = 1; args[count] != NULL; count++)
     {
-        if (strlen(logStr) + 1 + strlen(args[count]) < 1024)
+        if (strlen(logStr) + 1 + strlen(args[count]) < 2048)
         {
             strcat(logStr, " ");
             strcat(logStr, args[count]);
@@ -309,6 +318,14 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
     char trackstring[4];
     
     snprintf(trackstring, 4, "%d", tracknum);
+
+    // Evade cdparanoia's 245 char path limit by changing to the target
+    // directory (in exec_with_output()) and using a temporary file name
+    // with no absolute path.
+    char trackname[16];
+    snprintf(trackname, sizeof(trackname), "x%02d.wav", tracknum);
+    gchar * dir = g_path_get_dirname(filename);
+    gchar * xfilename = g_build_filename(dir, trackname, NULL);
     
     pos = 0;
     args[pos++] = "cdparanoia";
@@ -318,10 +335,10 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
     args[pos++] = "-d";
     args[pos++] = cdrom;
     args[pos++] = trackstring;
-    args[pos++] = filename;
+    args[pos++] = trackname;
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &cdparanoia_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &cdparanoia_pid, dir);
     
     // to convert the progress number stat cdparanoia spits out
     // into sector numbers divide by 1176
@@ -372,7 +389,17 @@ void cdparanoia(char * cdrom, int tracknum, char * filename, double * progress)
         debugLog("w3\n");
         usleep(100000);
     }
-    
+
+    int rc = rename(xfilename, filename);
+    if (rc == -1)
+    {
+        snprintf(logStr, 1024, "cdparanoia(): rename() to '%s' failed with errno=%d\n", filename, errno);
+        debugLog(logStr);
+    }
+    g_free(xfilename);
+    g_free(dir);
+
+    *progress = 1;
 }
 
 // uses LAME to encode a WAV file into a MP3 and tag it
@@ -468,7 +495,7 @@ void lame(int tracknum,
     args[pos++] = mp3filename;
     args[pos++] = NULL;
 
-    fd = exec_with_output(args, STDERR_FILENO, &lame_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &lame_pid, NULL);
     
     do
     {
@@ -484,7 +511,7 @@ void lame(int tracknum,
             if (size == -1 && errno == EINTR)
             /* signal interrupted read(), try again */
             {
-                size = 1;
+                pos--;
                 debugLog("lame() interrupted");
                 interrupted = TRUE;
             }
@@ -505,7 +532,7 @@ void lame(int tracknum,
         debugLog("w4\n");
         usleep(100000);
     }
-    
+    *progress = 1;
 }
 
 // uses oggenc to encode a WAV file into a OGG and tag it
@@ -585,7 +612,7 @@ void oggenc(int tracknum,
     args[pos++] = oggfilename;
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid, NULL);
     
     do
     {
@@ -626,6 +653,7 @@ void oggenc(int tracknum,
         debugLog("w6\n");
         usleep(100000);
     }
+    *progress = 1;
 }
 
 // uses opusenc to encode a WAV file into a OGG and tag it
@@ -704,7 +732,7 @@ void opusenc(int tracknum,
     args[pos++] = opusfilename;
     args[pos++] = NULL;
 
-    fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &oggenc_pid, NULL);
 
     int opussize;
     char opusbuf[256];
@@ -727,6 +755,7 @@ void opusenc(int tracknum,
         debugLog("w12\n");
         usleep(100000);
     }
+    *progress = 1;
 }
 
 
@@ -734,6 +763,7 @@ void opusenc(int tracknum,
 //
 // tracknum - the track number
 // artist - the artist's name
+// albumartist - we want to tag this if it is a compilation
 // album - the album the song came from
 // title - the name of the song
 // wavfilename - the path to the WAV file to encode
@@ -742,6 +772,8 @@ void opusenc(int tracknum,
 // progress - the percent done
 void flac(int tracknum,
           char * artist,
+          char * albumartist, //mw
+          gboolean single_artist, //mw
           char * album,
           char * title,
           char * genre,
@@ -761,12 +793,13 @@ void flac(int tracknum,
     
     char tracknum_text[16];
     char * artist_text = NULL;
+    char * albumartist_text = NULL; //mw
     char * album_text = NULL;
     char * title_text = NULL;
     char * genre_text = NULL;
     char * year_text = NULL;
     char compression_level_text[3];
-    const char * args[19];
+    const char * args[21];
     
     snprintf(tracknum_text, 16, "TRACKNUMBER=%d", tracknum);
     
@@ -778,6 +811,16 @@ void flac(int tracknum,
         snprintf(artist_text, strlen(artist) + 8, "ARTIST=%s", artist);
     }
     
+    //mw
+    if((albumartist != NULL) && (!single_artist))
+    {
+        albumartist_text = malloc(strlen(albumartist) + 13);
+        if (albumartist_text == NULL)
+            fatalError("malloc(sizeof(char) * (strlen(albumartist)+13)) failed. Out of memory.");
+        snprintf(albumartist_text, strlen(albumartist) + 13, "ALBUMARTIST=%s", albumartist);
+    }
+    //mw end
+    
     if(album != NULL)
     {
         album_text = malloc(strlen(album) + 7);
@@ -826,6 +869,15 @@ void flac(int tracknum,
         args[pos++] = "-T";
         args[pos++] = artist_text;
     }
+
+    //mw
+    if ((albumartist != NULL) && (!single_artist) && (strlen(albumartist) > 0))
+    {
+        args[pos++] = "-T";
+        args[pos++] = albumartist_text;
+    }
+    //mw end
+
     if ((album != NULL) && (strlen(album) > 0))
     {
         args[pos++] = "-T";
@@ -851,10 +903,11 @@ void flac(int tracknum,
     args[pos++] = flacfilename;
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &flac_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &flac_pid, NULL);
     
     free(artist_text);
     free(album_text);
+    free(albumartist_text);
     free(title_text);
     free(genre_text);
     free(year_text);
@@ -878,19 +931,10 @@ void flac(int tracknum,
                 interrupted = TRUE;
             }
             
-        } while ((size > 0 && pos < 255 && buf[pos] != '\r' && buf[pos] != '\n') || interrupted);
+        } while ((size > 0 && pos < 255 && buf[pos] != '\r' && buf[pos] != '\n' && buf[pos] != '\b') || interrupted);
         buf[pos] = '\0';
 
-        for (; pos>0; pos--)
-        {
-            if (buf[pos] == ':')
-            {
-                pos++;
-                break;
-            }
-        }
-
-        if (sscanf(&buf[pos], "%d%%", &sector) == 1)
+        if (sscanf(buf, "%d%% ", &sector) == 1)
         {
             *progress = (double)sector/100;
         }
@@ -904,20 +948,41 @@ void flac(int tracknum,
         debugLog("w7\n");
         usleep(100000);
     }
+    *progress = 1;
 }
 
 void wavpack(int tracknum,
              char* wavfilename,
+             char* wavpackfilename_wv,  // .wv file
+             char* wavpackfilename_wvc, // .wvc file, assumed to be in same dir as .wv file [if hybrid]
              int compression,
              bool hybrid,
              int bitrate,
              double* progress)
 {
-    const char* args[10];
+    char logStr[1024];
+    const char* args[12];
     int fd;
     int pos;
     int size;
     char buf[256];
+
+    // The wavpack executable creates temporary files longer than the given filename.
+    //
+    // Temp files are named "base.tmp.wv" and "base.tmp.wvc" [if hybrid].
+    //
+    // Since the output filename could be as long as is allowable, the
+    // temporary filename could be longer and hence non-allowable, so use the
+    // target directory and temporary names for the encoded files, then rename
+    // them afterwards.
+
+    char trackname_wv[16];
+    char trackname_wvc[16];
+    snprintf(trackname_wv, sizeof(trackname_wv), "wv%02d.wv", tracknum);
+    snprintf(trackname_wvc, sizeof(trackname_wvc), "wv%02d.wvc", tracknum);
+    gchar * dir = g_path_get_dirname(wavpackfilename_wv);
+    gchar * xwavpackfilename_wv = g_build_filename(dir, trackname_wv, NULL);
+    gchar * xwavpackfilename_wvc = g_build_filename(dir, trackname_wvc, NULL);
     
     pos = 0;
     args[pos++] = "wavpack";
@@ -943,9 +1008,11 @@ void wavpack(int tracknum,
     args[pos++] = "-x3";
     
     args[pos++] = wavfilename;
+    args[pos++] = "-o";
+    args[pos++] = trackname_wv;
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &wavpack_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &wavpack_pid, dir);
     
     do
     {
@@ -999,6 +1066,27 @@ void wavpack(int tracknum,
         debugLog("w8\n");
         usleep(100000);
     }
+
+    int rc = rename(xwavpackfilename_wv, wavpackfilename_wv);
+    if (rc == -1)
+    {
+        snprintf(logStr, 1024, "wavpack(): rename() to '%s' failed with errno=%d\n", wavpackfilename_wv, errno);
+        debugLog(logStr);
+    }
+    if (hybrid)
+    {
+        rc = rename(xwavpackfilename_wvc, wavpackfilename_wvc);
+        if (rc == -1)
+        {
+            snprintf(logStr, 1024, "wavpack(): rename() to '%s' failed with errno=%d\n", wavpackfilename_wvc, errno);
+            debugLog(logStr);
+        }
+    }
+
+    g_free(xwavpackfilename_wv);
+    g_free(xwavpackfilename_wvc);
+    g_free(dir);
+    *progress = 1;
 }
 
 void mac(char* wavfilename,
@@ -1021,7 +1109,7 @@ void mac(char* wavfilename,
     
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &monkey_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &monkey_pid, NULL);
     
     int size;
     char buf[256];
@@ -1057,6 +1145,7 @@ void mac(char* wavfilename,
         debugLog("w9\n");
         usleep(100000);
     }
+    *progress = 1;
 }
 
 void musepack(char* wavfilename,
@@ -1081,7 +1170,7 @@ void musepack(char* wavfilename,
     args[pos++] = musepackfilename;
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &musepack_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &musepack_pid, NULL);
     
     int size;
     char buf[256];
@@ -1121,6 +1210,7 @@ void musepack(char* wavfilename,
         debugLog("w10\n");
         usleep(100000);
     }
+    *progress = 1;
 }
 
 void aac(int tracknum,
@@ -1154,7 +1244,7 @@ void aac(int tracknum,
     args[pos++] = aacfilename;
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &aac_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &aac_pid, NULL);
     
     int size;
     char buf[256];
@@ -1213,7 +1303,7 @@ void aac(int tracknum,
 
     args[pos++] = NULL;
     
-    fd = exec_with_output(args, STDERR_FILENO, &aac_pid);
+    fd = exec_with_output(args, STDERR_FILENO, &aac_pid, NULL);
     
     do
     {
@@ -1237,4 +1327,5 @@ void aac(int tracknum,
     {
         free(dynamic_args[--dyn_pos]);
     }
+    *progress = 1;
 }
