@@ -39,9 +39,10 @@ void sigchld(int signum);
 // args - a valid array for execvp()
 // toread - the file descriptor to pipe back to the parent
 // p - a place to write the PID of the exec'ed process
+// dir - directory to run program in
 // 
 // returns - a file descriptor that reads whatever the program outputs on "toread"
-int exec_with_output(const char * args[], int toread, pid_t * p);
+int exec_with_output(const char * args[], int toread, pid_t * p, const char * dir);
 
 // uses cdparanoia to rip a WAV track from a cdrom
 //
@@ -120,6 +121,7 @@ void opusenc(int tracknum,
 //
 // tracknum - the track number
 // artist - the artist's name
+// albumartist - we want to tag this if it is a compilation
 // album - the album the song came from
 // title - the name of the song
 // wavfilename - the path to the WAV file to encode
@@ -128,6 +130,8 @@ void opusenc(int tracknum,
 // progress - the percent done
 void flac(int tracknum,
           char * artist,
+          char * albumartist, //mw
+          gboolean single_artist, //mw
           char * album,
           char * title,
           char * genre,
@@ -139,6 +143,8 @@ void flac(int tracknum,
 
 void wavpack(int tracknum,
              char * wavfilename,
+             char * wavpackfilename_wv,
+             char * wavpackfilename_wvc,
              int compression,
              bool hybrid,
              int bitrate,
